/* CakeOS */
#include <system.h>

isr_t interrupt_handlers[256];

void register_interrupt_handler(u8int n, isr_t handler)
{
    interrupt_handlers[n] = handler;
}

char *exception_messages[] =
    /// Just A fancy Array of charecters here
    /// We use this to identify the intel Exceptions to the kernel
{
    "Division By Zero",
    "Debug",
    "Non Maskable Interrupt",
    "Breakpoint",
    "Into Detected Overflow",
    "Out of Bounds",
    "Invalid Opcode",
    "No Coprocessor",

    "Double Fault",
    "Coprocessor Segment Overrun",
    "Bad TSS",
    "Segment Not Present",
    "Stack Fault",
    "General Protection Fault",
    "Page Fault",
    "Unknown Interrupt",

    "Coprocessor Fault",
    "Alignment Check",
    "Machine Check",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",

    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved"
};

// This gets called from our ASM interrupt handler stub.
void isr_handler(struct regs *r)
{
    if (r->int_no < 32)
    {
        puts(exception_messages[r->int_no]);
        puts(" Exception. System Halted!\n");
   	__asm__ __volatile__ (" movb $0x20, %al;  outb %al, $0x20");
//		outportb(0x20,0x20);
   	__asm__ __volatile__ ("sti");
     puts(" gs       fs       es      ds\n");
       put_hex32(r->gs); put_hex32(r->fs); put_hex32(r->es);
		 put_hex32(r->ds);

   puts("\n\n edi      esi      ebp      esp      ebx      edx      ecx      eax\n");
		put_hex32(r->edi); put_hex32(r->esi); put_hex32(r->ebp);
      put_hex32(r->esp); put_hex32(r->ebx); put_hex32(r->edx);
	   put_hex32(r->ecx); put_hex32(r->eax);


   puts("\n\n int_no     err_code\n");
   put_hex32(r->int_no); put_hex32(r->err_code);

 puts("\n\n eip      cs       eflags   useresp  ss\n");
      put_hex32(r->eip); put_hex32(r->cs); put_hex32(r->eflags);
      put_hex32(r->useresp); put_hex32(r->ss);
       puts("\n");

       for (;;){ 	__asm__ __volatile__ ("hlt"); }
		  ABORT();
    }
}

// This gets called from our ASM interrupt handler stub.
void irq_handler(registers_t regs)
{   
    // Send an EOI (end of interrupt) signal to the PICs.
    // If this interrupt involved the slave.
    if (regs.int_no >= 40)
    {
        // Send reset signal to slave.
        outb(0xA0, 0x20);
    }
    // Send reset signal to master.
    outb(0x20, 0x20);

    if (interrupt_handlers[regs.int_no] != 0)
    {   
        isr_t handler = interrupt_handlers[regs.int_no];
        handler(&regs);
    }
}
